#!/bin/bash

PATH="/bin:/sbin:/usr/bin:/usr/sbin:/usr/X11R6/bin:/usr/local/bin"
export PATH

XDIALOG_HIGH_DIALOG_COMPAT=1
export XDIALOG_HIGH_DIALOG_COMPAT

XDIALOG_NO_GMSGS=1
export XDIALOG_NO_GMSGS

XTERM=rxvt
type -a rxvt >/dev/null 2>&1 || XTERM=xterm

# Get root
[ "`id -u`" != "0" ] && exec sudo "$0" "$@"

TMP="/tmp/gprsconnect$$"

bailout(){
rm -f "$TMP"
exit 0
}

DIALOG="dialog"
[ -n "$DISPLAY" ] && [ -x /usr/bin/Xdialog ] && DIALOG="Xdialog"
trap bailout 1 2 3 15

# LANGUAGE etc.
[ -f /etc/sysconfig/knoppix ] && . /etc/sysconfig/knoppix
[ -z "$LANG" ] && export LANG
[ -z "$LANGUAGE" ] && export LANGUAGE
[ -z "$CHARSET" ] && export CHARSET

# Language-dependent Messages
case "$LANGUAGE" in
de*|at*|ch*)
TITLE0="Keine Schnittstelle angegeben"
MESSAGE0="Sie haben die Schnittstelle noch nicht eingerichtet, über die das Modem oder das Handy erreichbar ist. Möchten Sie dies jetzt nachholen?"
TITLE_PROVIDER="GPRS/UMTS - Provider"
MESSAGE_PROVIDER="Bitte geben Sie an, welchen Mobilfunkanbieter Sie verwenden:"
TITLE_TAG="GPRS/UMTS - Provider-String"
MESSAGE_TAG="Bitte geben Sie den INIT-String ein, den Ihr Provider zur Nutzung von GPRS/UMTS in seiner Dokumentation empfiehlt:"
TITLE_CPIN="PIN"
MESSAGE_CPIN="Falls zum Entsperren der GPRS/UMTS-Karte eine PIN-Nummer erforderlich ist, geben Sie diese bitte hier an:"
NOCHANGE="(Keine Änderung)"
TITLE1="GPRS/UMTS"
MESSAGE1="Bitte beachten Sie, dass je nach Anbieter und Vertrag für die Nutzung von GPRS/UMTS hohe Volumenkosten anfallen können (üblicherweise ca. 1 cent/kB), wobei GPRS/UMTS ansonsten unabhängig von der Online-Zeit ist. Eine Traffic-Statistik können Sie in der \"Root-Shell\" mit dem Programm iptraf erhalten (über ppp0). GPRS/UMTS Internet-Zugang jetzt starten?"
TITLE_LOG="Starte GPRS/UMTS Einwahl, Beenden mit Steuerung-C."
MESSAGE_CONNECT="Verbindung starten"
MESSAGE_DISCONNECT="Verbindung beenden"
MESSAGE_SETTINGS="Konfiguration einstellen/ändern"
MESSAGE_FAILURE="Verbindung fehlgeschlagen."
MESSAGE_SUCCESS="Verbindung erfolgreich initiiert."
;;
*)
TITLE0="No modem device selected"
MESSAGE0="You have not configured a device for modem / cellphone access yet. Would you like to do this now?"
TITLE_PROVIDER="GPRS/UMTS - Provider"
MESSAGE_PROVIDER="Please select your cellphone provider:"
TITLE_TAG="GPRS/UMTS - Provider-String"
MESSAGE_TAG="Please enter the INIT string for GPRS/UMTS that your provider recommends (check your documentation):"
TITLE_CPIN="PIN"
MESSAGE_CPIN="If this is a GPRS/UMTS card that needs to be unlocked with a PIN, please enter this pin here:"
NOCHANGE="(No Change)"
TITLE1="GPRS/UMTS"
MESSAGE1="Please be aware that surfing the net over GPRS/UMTS, while not being onlinetime-dependent, can cause high costs for traffic volume, depending on your provider (usually about 1 cent/kB). You can get a detailled statistic about internet traffic in the \"Root-Shell\" using the program iptraf (on ppp0). Start GPRS/UMTS Internet Access now?"
TITLE_LOG="Dialup initiated, hit Ctrl-C or close window to disconnect."
MESSAGE_CONNECT="Start connection"
MESSAGE_DISCONNECT="Disconnect"
MESSAGE_SETTINGS="Edit/change configuration"
MESSAGE_FAILURE="Connection failed."
MESSAGE_SUCCESS="Connection established successfully."
;;
esac

PROVIDERS=(default "$NOCHANGE" on \
o "O2" off \
e "E-Plus" off \
v "Vodafone" off \
t "T-Mobile" off \
other "(INIT-String)" off)

dial(){
 [ -c /dev/ppp ] || mknod -m 660 /dev/ppp c 108 0
 if [ -n "$DISPLAY" ]; then
  $XTERM -T "$TITLE_LOG" -e pppd updetach call gprs
  if ifconfig ppp0 >/dev/null 2>&1; then
   $DIALOG --clear --title "$TITLE1" --msgbox "$MESSAGE_SUCCESS" 8 35
   return 0
  else
   $DIALOG --clear --title "$TITLE1" --msgbox "$MESSAGE_FAILURE" 8 35
   return 1
  fi
 else
  # Start pppd right here.
  pppd updetach call gprs
  if ifconfig ppp0 >/dev/null 2>&1; then
   read -p "$MESSAGE_SUCCESS" -t 10 dummy
   return 0
  else
   read -p "$MESSAGE_FAILURE" -t 30 dummy
   return 1
  fi
 fi
}

configure(){
 rm -f "$TMP"
 $DIALOG --clear --title "$TITLE_PROVIDER" --radiolist "$MESSAGE_PROVIDER" 18 75 12 "${PROVIDERS[@]}" 2>"$TMP" || bailout 1
 read PROVIDER <"$TMP";  rm -f "$TMP"
 PROVIDER="${PROVIDER#\"}"; PROVIDER="${PROVIDER%\"}"
 INIT=""
 case "$PROVIDER" in
  default) ;;
  o) INIT='AT+CGDCONT=1,"IP","internet"';;
  e) INIT='AT+CGDCONT=1,"IP","internet.eplus.de"' ;;
  v) INIT='AT+CGDCONT=1,"IP","web.vodafone.de"' ;;
  t) INIT='AT+CGDCONT=1,"IP","internet.t-d1.de"' ;;
  *) rm -f "$TMP"; $DIALOG --title "$TITLE_TAG" --inputbox "$MESSAGE_TAG" 12 60 "AT+CGDCONT=1,\"IP\",\"internet\"" 2>"$TMP" || bailout 1
     read INIT <"$TMP"; rm -f "$MP" ;;
 esac
 # Skip if already configured, i.e. $INIT is empty.
 if [ -n "$INIT" -o ! -s /etc/chatscripts/gprs ]; then
  rm -f "$TMP"
  $DIALOG --clear --title "$TITLE_CPIN" --inputbox "$MESSAGE_CPIN" 12 55 "$CPIN" 2>"$TMP" || bailout 1
  read CPIN <"$TMP"; rm -f "$TMP"
  CPIN="${CPIN#\"}"; CPIN="${CPIN%\"}"
  rm -f /etc/chatscripts/gprs
  if [ -n "$CPIN" ]; then
   # PIN version
   sed 's/@@CPIN@@/'"$CPIN"'/g' /etc/chatscripts/gprs-pin > /etc/chatscripts/gprs
  else
   cp /etc/chatscripts/gprs-nopin /etc/chatscripts/gprs
  fi
  chown root.dip /etc/chatscripts/gprs
  chmod 660 /etc/chatscripts/gprs
  [ "$PROVIDER" = "default" -o -z "$INIT" ] || perl -pi -e 's/AT\+CGDCONT=.*$/'"$INIT"'/g' /etc/chatscripts/gprs
 fi
}

# MAIN
[ "$1" = "reconnect" ] && dial && bailout 0

if [ ! -c /dev/modem ]; then
 if [ -r /etc/sysconfig/modemlink ]; then
  . /etc/sysconfig/modemlink
  [ -n "$MODEMLINK" ] && [ -c "$MODEMLINK" ] && ln -snf "$MODEMLINK" /dev/modem
 elif [ -r /etc/adriane/adriane.conf -o -r "$HOME"/.adriane/adriane.conf ]; then
  [ -r /etc/adriane/adriane.conf ] && . /etc/adriane/adriane.conf
  [ -r "$HOME"/.adriane/adriane.conf ] && . "$HOME"/.adriane/adriane.conf
  [ -n "$DEVICE_GPRS" ] && [ -c "$DEVICE_GPRS" ] && [ "$DEVICE_GPRS" != "/dev/modem" ] && ln -snf "$DEVICE_GPRS" /dev/modem
 fi
fi

if [ ! -c /dev/modem ]; then
 $DIALOG --title "$TITLE0" --yesno "$MESSAGE0" 8 65 || bailout
 /usr/sbin/modemlink
 [ -c /dev/modem ] || bailout
fi

while true; do
 GPRSPID=""
 while read pid args; do
  echo "$args" | grep -q gprs && GPRSPID="$GPRSPID $pid"
 done <<EOT
$(ps --no-headers -C pppd -o pid,args)
EOT
 if [ -n "$GRSPID" ]; then
  ACTIONS=(d "$MESSAGE_DISCONNECT")
 elif [ -s /etc/chatscripts/gprs ]; then
  ACTIONS=(c "$MESSAGE_CONNECT" e "$MESSAGE_SETTINGS")
 else
  ACTIONS=(e "$MESSAGE_SETTINGS")
 fi
 rm -f "$TMP"
 $DIALOG --clear --title "GPRS" --menu " " 14 75 8 "${ACTIONS[@]}" 2>"$TMP" || bailout 0
 read selection <"$TMP"
 case "$selection" in
  d) kill $GPRSPID ;;
  c) dial && break ;;
  e) configure || break ;;
 esac
done

